@@ -13,10 +13,11 @@
 # limitations under the License.
 
 import os
+import typing as T
 
 logfile = 'meson-logs/install-log.txt'
 
-def do_uninstall(log):
+def do_uninstall(log: str) -> None:
     failures = 0
     successes = 0
     for line in open(log):
@@ -24,18 +25,21 @@ def do_uninstall(log):
             continue
         fname = line.strip()
         try:
-            os.unlink(fname)
+            if os.path.isdir(fname) and not os.path.islink(fname):
+                os.rmdir(fname)
+            else:
+                os.unlink(fname)
             print('Deleted:', fname)
             successes += 1
         except Exception as e:
-            print('Could not delete %s: %s.' % (fname, e))
+            print(f'Could not delete {fname}: {e}.')
             failures += 1
     print('\nUninstall finished.\n')
     print('Deleted:', successes)
     print('Failed:', failures)
     print('\nRemember that files created by custom scripts have not been removed.')
 
-def run(args):
+def run(args: T.List[str]) -> int:
     if args:
         print('Weird error.')
         return 1
